import 'clx_notification_body.dart';
import 'clx_notification_name.dart';

final defaultNotificationCenter = _CLXNotificationCenter.getInstance();

final class _CLXNotificationCenter {
  static _CLXNotificationCenter? _instance;

  _CLXNotificationCenter._internal();

  static _CLXNotificationCenter getInstance() {
    return _instance ?? _CLXNotificationCenter._internal();
  }

  /// 消息管理器
  final Set<CLXNotificationBody> _notificationItems = {};

  /// 注册消息监听
  void registerNotification(
      {required String name, required CLXNotificationChannel callback}) {
    _notificationItems.firstWhere((element) => element.name.isEqual(name),
        orElse: () {
      CLXNotificationBody body =
          CLXNotificationBody(name: CLXNotificationName(name: name));
      _notificationItems.add(body);
      return body;
    }).addListen(callback);
  }

  /// 移除消息监听
  void removeNotification(
      {required String name, required CLXNotificationChannel callback}) {
    _notificationItems
        .firstWhere((element) => element.name.isEqual(name))
        .removeListen(callback);
  }

  /// 移除所有消息监听
  void removeNotificationServer({required String name}) {
    final notif =
        _notificationItems.firstWhere((element) => element.name.isEqual(name));
    _notificationItems.remove(notif);
  }

  /// 移除所有消息监听
  void removeAllNotification() {
    _notificationItems.clear();
  }

  /// 发送消息
  Future<bool> sendNotification(
      {required String name,
      dynamic object,
      Duration delay = Duration.zero}) async {
    if (_notificationItems.isEmpty) {
      return Future.value(false);
    }

    if (delay != Duration.zero) {
      await Future.delayed(delay);
    }
    final list =
        _notificationItems.where((element) => element.name.isEqual(name));
    final resList = list.map((e) => e.notify(object: object)).toList();
    return resList.isNotEmpty;
  }
}
